/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, document, tau, console*/

/**
 * Details view module.
 */
define({
    name: 'views/details',
    requires: [
        'core/event',
        'models/apps',
        'helpers/uimenu'
    ],
    def: function main(event, apps, uiMenu) {
        'use strict';

        /**
         * Reference to the title field.
         * @type {HTMLElement}
         */
        var title = null,

            /**
             * Reference to the icon element.
             * @type {HTMLElement}
             */
            icon = null,

            /**
             * Reference to the version element.
             * @type {HTMLElement}
             */
            version = null,

            /**
             * Reference to the install date element.
             * @type {HTMLElement}
             */
            installDate = null,

            /**
             * Reference to the app size element.
             * @type {HTMLElement}
             */
            appSize = null,

            /**
             * Reference to the page element.
             * @type {HTMLElement}
             */
            page = null,

            /**
             * Reference to the page content div.
             * @type {HTMLElement}
             */
            content = null,

            /**
             * Reference to the scroller element on the circular page.
             * @type {HTMLElement}
             */
            scroller = null,

            /**
             * Reference to the message popup.
             * @type {HTMLElement}
             */
            popupMessage = null,

            /**
             * Displayed app.
             * @type {ApplicationInformation}
             */
            app = null;

        /**
         * Returns the specified date formatted.
         * @param {Date} date
         * @returns {string}
         */
        function formatDate(date) {
            return date.toISOString().replace('T', '\n').substring(0, 19);
        }

        /**
         * Returns the specified count of bytes formatted.
         * @param {number} size
         * @returns {string}
         */
        function formatSize(size) {
            var units = ['B', 'kB', 'MB', 'GB'],
                i = 0,
                value = 0;

            while (size > Math.pow(1024, i + 1)) {
                i += 1;
            }

            if (i >= units.length) {
                i = units.length - 1;
            }

            value = (size / Math.pow(1024, i));

            if (i > 0) {
                value = value.toFixed(2);
            }

            return value + ' ' + units[i];
        }

        /**
         * Handles the views.main.clicked event.
         * @param {object} e Event data.
         */
        function onAppClicked(e) {
            app = apps.getApp(e.detail);
            title.innerText = app.name;
            icon.src = app.iconPath;
            version.innerText = app.version;
            installDate.innerText = formatDate(app.installDate);
            appSize.innerText = app.size ? formatSize(app.size) : 'unknown';
        }

        /**
         * Displays a popup with the specified message.
         * @param {string} message
         */
        function showMessageToast(message) {
            popupMessage.innerText = message;
            tau.openPopup('#popup-message');
        }

        /**
         * Handles launch success event.
         */
        function onLaunchSuccess() {
            showMessageToast('Application launched');
        }

        /**
         * Handles launch error event.
         */
        function onLaunchError(error) {
            showMessageToast('Application could not be launched');
            console.error('Launch error: ' + error.message, error);
        }

        /**
         * Handles click events on the launch menu item.
         */
        function onLaunchClick() {
            apps.launch(app.id, onLaunchSuccess, onLaunchError);
        }

        /**
         * Handles the 'pagebeforeshow' event.
         */
        function onPageBeforeShow() {
            var menu = uiMenu.createUiMenu('details', 'details-menu-popup');
            menu.bindClickEvent('launch', onLaunchClick);
        }

        /**
         * Handles the 'pageshow' event.
         */
        function onPageShow() {
            if (tau.support.shape.circle && !scroller) {
                scroller = page.querySelector('.ui-scroller');
            }
            if (scroller) {
                scroller.scrollTop = 0;
            } else {
                content.scrollTop = 0;
            }
        }

        /**
         * Initializes the module.
         */
        function init() {
            page = document.getElementById('details');
            title = document.querySelector('#details header h2');
            icon = document.getElementById('icon');
            version = document.getElementById('version');
            installDate = document.getElementById('install-date');
            appSize = document.getElementById('app-size');
            content = page.querySelector('.ui-content');
            popupMessage = document.getElementById('popup-message-text');

            event.listeners({
                'views.main.clicked': onAppClicked
            });
            page.addEventListener('pagebeforeshow', onPageBeforeShow);
            page.addEventListener('pageshow', onPageShow);

            icon.addEventListener('error', function onIconLoadError() {
                icon.style.display = 'none';
            });

            icon.addEventListener('load', function onIconLoadSuccess() {
                icon.style.display = 'block';
            })
        }

        return {
            init: init
        };
    }
});
